//======================================================== file = arbit.c =====
//=  Arbitrators for VOQ switch simulation model                              =
//=   - Implements priority iSLIP only                                        =
//=============================================================================
//=  Notes: 1) "2" is higher priority than "1"                                =
//=---------------------------------------------------------------------------=
//=  Contains: arbit_islip() - Implements priority iSLIP arbitration          =
//=---------------------------------------------------------------------------=
//=  History:  KJC/KY (02/20/01) - Genesis (from voq_1 1.00)                  =
//=            KY (3/14/01) - Fixed PAUSE                                     =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//================================================================= ISLIP =====
//= Implements priority iSLIP arbitration                                     =
//=============================================================================
void arbit_islip(void)
{
  int      request[N][N];    // Request array (IP to OP)
  int      grant[N][N];      // Grant array (OP to IP)
  int      accept[N][N];     // Accept array (IP to OP)
  int      op_commit[N];     // TRUE iff OP i is committed (matched)
  int      ip_commit[N];     // TRUE iff IP i is committed (matched)
  int      next_ip, next_op; // Next values for ip and op
  int      ip, op;           // Counters for IP and OP
  int      g1[N], a1[N];     // iSLIP "g" and "a" pointers - priority 0 cells
  int      g2[N], a2[N];     // iSLIP "g" and "a" pointers - priority 1 cells
  int      iter;             // Counter for iterations

  create("arbit_islip");

  // Initialize g1[], a1[], g2[], and a2[]
  for (ip=0; ip<N; ip++)
    for (op=0; op<N; op++)
      g1[op] = a1[ip] = g2[op] = a2[ip] = 0;

  // Arbitrate the ports forever
  while(1)
  {
    // Clear accept[][], ip_commit[], and op_commit[]
    for (ip=0; ip<N; ip++)
      for (op=0; op<N; op++)
        accept[ip][op] = ip_commit[ip] = op_commit[op] = FALSE;

    // Do for NUM_ITER
    for (iter=0; iter<NUM_ITER; iter++)
    {
      // Load request array from Q_event_0[][] and Q_event_1[][] for
      // un-matched IP's
      for (ip=0; ip<N; ip++)
        for (op=0; op<N; op++)
          if ((queue_cnt(Q_event_1[ip][op]) > 0) && (ip_commit[ip] == FALSE))
            request[ip][op] = 2;
          else if ((queue_cnt(Q_event_0[ip][op]) > 0) &&
                   (ip_commit[ip] == FALSE))
            request[ip][op] = 1;
          else
            request[ip][op] = 0;

      // Clear grant[][]
      for (ip=0; ip<N; ip++)
        for (op=0; op<N; op++)
          grant[ip][op] = 0;

      // For all OP's do grants to IP's
      for (op=0; op<N; op++)
      {
        // Check if this OP is committed (matched)
        if (op_commit[op] == TRUE) continue;

        // Set grant[][] according to iSLIP
        // *** this for priority 1
        next_ip = g2[op];
        for (ip=0; ip<N; ip++)
        {
          if (request[next_ip][op] == 2)
          {
            grant[next_ip][op] = 2;
            goto skip1;
          }
          next_ip = (next_ip + 1) % N;
        }
        // *** this for priority 0
        next_ip = g1[op];
        for (ip=0; ip<N; ip++)
        {
          if (request[next_ip][op] == 1)
          {
            grant[next_ip][op] = 1;
            goto skip1;
          }
          next_ip = (next_ip + 1) % N;
        }
        skip1:
      }

      // For all IP's do accepts to OP's
      for (ip=0; ip<N; ip++)
      {
        // Set accept[] according to iSLIP
        // *** this for priority 1
        next_op = a2[ip];
        for (op=0; op<N; op++)
        {
          if (grant[ip][next_op] == 2)
          {
            accept[ip][next_op] = 2;
            ip_commit[ip] = TRUE;
            op_commit[next_op] = TRUE;
            request[ip][next_op] = 0;
            if (iter == 0)
            {
              a2[ip] = (next_op + 1) % N;
              g2[next_op] = (ip + 1) % N;
            }
            goto skip2;
          }
          next_op = (next_op + 1) % N;
        }
        // *** this for priority 0
        next_op = a1[ip];
        for (op=0; op<N; op++)
        {
          if (grant[ip][next_op] == 1)
          {
            accept[ip][next_op] = 1;
            ip_commit[ip] = TRUE;
            op_commit[next_op] = TRUE;
            request[ip][next_op] = 0;
            if (iter == 0)
            {
              a1[ip] = (next_op + 1) % N;
              g1[next_op] = (ip + 1) % N;
            }
            goto skip2;
          }
          next_op = (next_op + 1) % N;
        }
        skip2:
      }
    }

    // Fire the cells for all accept[][]
    for (ip=0; ip<N; ip++)
      for (op=0; op<N; op++)
        if (accept[ip][op] > 0)
        {
          set(Inport_busy[ip]);
          set(Outport_busy[op]);
          if (accept[ip][op] == 2) set(Q_event_1[ip][op]);
          if (accept[ip][op] == 1) set(Q_event_0[ip][op]);
        }

    // Hold for a cell time and then an empty hold for synchronization
    hold((8.0 * CELL_SIZE)/(SPEED_UP * MEDIA_RATE));
    hold(0.0);
  }

  return;
}
