//======================================================= file = common.h =====
//=  Include file for VOQ switch simulation model                             =
//=============================================================================
//=  Notes:  1) This file contains                                            =
//=               - Constants                                                 =
//=               - Common (external) variables                               =
//=               - Common (external) CSIM types                              =
//=               - Function prototypes                                       =
//=          2)  Basic time unit is the millisecond                           =
//=---------------------------------------------------------------------------=
//=  History:  KJC/KY (02/20/01) - Genesis (from voq_1 1.00)                  =
//=            KY (3/14/01) - Fixed PAUSE                                     =
//=============================================================================
#pragma warn -pro                     // Disable function prototype warning
#pragma warn -stu                     // Disable undefined structure warning
#pragma warn -rch                     // Disable unreachable code warning
#pragma warn -ccc                     // Disable condition always true warning

//----- Conditional definitions -----------------------------------------------
#if defined(MAIN)                     // Determine if main or external
  #define EXTERN
#endif
#if defined(EXTERNAL)
  #define EXTERN extern
#endif

//----- Constants -------------------------------------------------------------
#define VERSION "1.00xp <03/19/01>"   // Version number
#define OUT_FILE       "result.txt"   // Name of output file

#define FALSE                   0     // FALSE = 0
#define TRUE                    1     // TRUE = 1
#define XOFF                    0     // PAUSE XOFF = 0
#define XON                     1     // PAUSE XON = 1
#define PROP_SPEED         4.5e-6     // Speed of light in meters/millsec

#define MEDIA_RATE         10.0e6     // Media rate in bits per millisec
#define CELL_SIZE             100     // Switch core cell size in bytes
#define SPEED_UP                1     // Switch core speed-up factor
#define N                      16     // Number of input ports in the switch
#define NUM_ITER                4     // Number of iSLIP iterations
#define IN_BUF_SIZE         65536     // Input buffer size per port in bytes
#define THRESH_LOW          16384     // Input buffer low threshold in bytes
#define THRESH_HIGH         49152     // Input buffer high threshold in bytes
#define LINK_LEN                0     // Link length in meters
#define GEN_FLAG                1     // 1 = gen_poisson1(), 2 = gen_poisson2()
                                      // 3 = for gen_ipp(), 4 = gen_cbr(),
                                      // 5 = gen_square()

//----- Overall external variables --------------------------------------------
EXTERN double    Sim_time;            // Simuation time
EXTERN int       Inport_count[N];     // Assertable cell counter semaphore
EXTERN int       Outport_count[N];    // Assertable outport counter semaphore
EXTERN int       Level[N];            // Level of an input queue in bytes
EXTERN int       Pause_sent[N];       // Current PAUSE state at switch
EXTERN int       Pause_recv[N];       // Current PAUSE state at generator
EXTERN int       Lost_count[N];       // Lost packet counter
EXTERN int       Pause_count[N];      // PAUSE sent counter
EXTERN int       Pr_voq[N];           // Priority packets in an input port

//----- External variables for gen_poisson() ----------------------------------
EXTERN double    Lambda1;             // Arrival rate of packets in pkts/ms

//----- External variables for gen_ipp() --------------------------------------
EXTERN double    Lambda2;             // Arrival rate of packets in pkts/ms
EXTERN double    Alpha;               // Transition rate from "off" to "on"
EXTERN double    Beta;                // Transition rate from "on" to "off"

//----- External variables for gen_cbr() --------------------------------------
// None (hardwired in gen_cbr())

//----- External variables for gen_square() -----------------------------------
// None (hardwired in gen_square())

//----- CSIM types ------------------------------------------------------------
EXTERN FACILITY Outport_buf[N];       // Outport buffer for reassembled packets
EXTERN EVENT    Q_event_0[N][N];      // Inport VOQs for packets - priority 0
EXTERN EVENT    Q_event_1[N][N];      // Inport VOQs for packets - priority 1
EXTERN EVENT    Inport_busy[N];       // Event for segmentation busy
EXTERN EVENT    Outport_busy[N];      // Event for outport busy
EXTERN EVENT    Xmit_done[N];         // Event for transmit done at outport
EXTERN EVENT    Pause_event[N];       // PAUSE event
EXTERN TABLE    Delay_table_0[N];     // Delay table for packet delay (pri 0)
EXTERN TABLE    Delay_table_1[N];     // Delay table for packet delay (pri 1)
EXTERN TABLE    Bytes_table[N];       // Bytes transfered table

//----- Function prototypes ---------------------------------------------------
// Traffic generators and source buffer
EXTERN void gen_poisson1(int in_id);
EXTERN void gen_poisson2(int in_id);
EXTERN void gen_ipp(int in_id);
EXTERN void gen_cbr(int in_id);
EXTERN void gen_square(int in_id);
EXTERN void (* gen[N])(int in_id);

// Links for propagation delay
EXTERN void send_pkt(int in_id, int out_id, int pkt_len, int pkt_type,
                     double org_time);
EXTERN void (* s_pkt[N])(int in_id, int out_id, int pkt_len, int pkt_type,
                         double org_time);
EXTERN void send_pause(int in_id, int pause_flag);
EXTERN void (* s_pause[N])(int in_id, int pause_flag);

// Segmenters and input ports
EXTERN void segmenter(int in_id, int out_id, int pkt_len, int pkt_type,
                      double org_time);
EXTERN void (* seg[N])(int in_id, int out_id, int pkt_len, int pkt_type,
                       double org_time);
EXTERN void inport(int in_id, int out_id, int pkt_len, int pkt_type,
                   int tail_flag, double org_time);
EXTERN void (* in[N])(int in_id, int out_id, int pkt_len, int pkt_type,
                      int tail_flag, double org_time);

// Output ports
EXTERN void outport(int from_id, int out_id, int pkt_len, int pkt_type,
                    int tail_flag, double org_time);
EXTERN void (* out[N])(int from_id, int out_id, int pkt_len, int pkt_type,
                       int tail_flag, double org_time);

// Switch arbitrator iSLIP
EXTERN void arbit_islip(void);

// Run-time simulation status reporter
EXTERN void run_sim(double run_time, int run_num);
