//======================================================= file = inport.c =====
//=  VOQ input port for VOQ switch simulation model                           =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: segmenter() - cell segmenter for input port                    =
//=            inport() - input port                                          =
//=---------------------------------------------------------------------------=
//=  History:  KJC/KY (02/20/01) - Genesis (from voq_1 1.00)                  =
//=            KY (3/14/01) - Fixed PAUSE                                     =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//============================================================================
//==  Segmenter for input port                                              ==
//============================================================================
void segmenter(int in_id, int out_id, int pkt_len, int pkt_type,
               double org_time)
{
  int    tail_flag;     // Flag for last packet in a cell
  int    num_cells;     // Number of cells in a packet
  int    i;             // Loop counter

  create("segmenter");

  // If no room for packet, discard it
  if (pkt_len > (IN_BUF_SIZE - Level[in_id]))
  {
    Lost_count[in_id]++;
    return;
  }

  // Increment priority counter if appropriate
  if (pkt_type == 1) Pr_voq[in_id]++;

  // Increase the level of this port buffer
  Level[in_id] = Level[in_id] + pkt_len;

  // Send PAUSE = XOFF if over high threshold and have not done so already
  if ((Level[in_id] > THRESH_HIGH) && (Pause_sent[in_id] == XON))
  {
    Pause_sent[in_id] = XOFF;
    s_pause[in_id](in_id, XOFF);
  }

  // Segment a packet into cells and put "cell-ized" packet in source buffer
  tail_flag = FALSE;
  num_cells = pkt_len / CELL_SIZE;
  if ((pkt_len % CELL_SIZE) > 0) num_cells = num_cells + 1;
  for(i=0; i<num_cells; i++)
  {
    if (i == (num_cells - 1)) tail_flag = TRUE;
    in[in_id](in_id, out_id, pkt_len, pkt_type, tail_flag, org_time);
  }
}

//============================================================================
//==  Input port                                                            ==
//============================================================================
void inport(int in_id, int out_id, int pkt_len, int pkt_type, int tail_flag,
            double org_time)
{
  create("inport");

  // Queue the arriving cell in the appropriate VOQ buffer
  if (pkt_type == 0) queue(Q_event_0[in_id][out_id]);
  if (pkt_type == 1) queue(Q_event_1[in_id][out_id]);

  // Assert that only one transmission occurs at an inport and outport
  assert(Inport_count[in_id] == 0);
  assert(Outport_count[out_id] == 0);

  // Increment counter semphores for an inport and outport pair
  Inport_count[in_id]++;
  Outport_count[out_id]++;

  // Decrease the level of this port buffer if last cell of a packet and
  // send PAUSE = XON if below low threshold and have not done so already
  if (tail_flag == TRUE)
  {
    Level[in_id] = Level[in_id] - pkt_len;
    if ((Level[in_id] < THRESH_LOW) && (Pause_sent[in_id] == XOFF))
    {
      Pause_sent[in_id] = XON;
      s_pause[in_id](in_id, XON);
    }
  }

  // Forward the cell to the outport
  hold((8.0 * CELL_SIZE) / (SPEED_UP * MEDIA_RATE));
  out[out_id](in_id, out_id, pkt_len, pkt_type, tail_flag, org_time);

  // Update Pr_voq as appropriate if last cell of a packet
  if ((pkt_type == 1) && (tail_flag == TRUE))
  {
    Pr_voq[in_id]--;
    assert(Pr_voq[in_id] >= 0);
  }
}
